@@ -1,17 +1,21 @@
 /*****************************************************/
 /** PREDICT installer program by KD2BD October 1999 **/
 /**   Updated to use .h files on December 27, 1999  **/
+/**  Changed the way soundcard is probed:  Feb 2000 **/
 /*****************************************************/
 
 #include <curses.h>
 #include <unistd.h>
 #include <string.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <linux/soundcard.h>
 
 int main()
 {
 	char version[10], pwd[80], line[255], ans;
-	int *system(), *cc;
-	FILE *infile, *outfile, *soundcard;
+	int *system(), *cc, dsp=-1;
+	FILE *infile, *outfile;
 
 	initscr();
 	start_color();
@@ -27,10 +31,10 @@ int main()
 
 	cc=0;
 	getcwd(pwd,79);
-	soundcard=fopen("/dev/dsp","r");
+	dsp=open("/dev/dsp",O_WRONLY);
 
-	if (soundcard!=NULL)
-		fclose(soundcard);
+	if (dsp!=-1)
+		close(dsp);
 
 	infile=fopen(".version","r");
 	fscanf(infile,"%s",version);
@@ -43,7 +47,7 @@ int main()
 	mvprintw(3,18,"                                           ");
 	mvprintw(4,18,"         --== PREDICT  v%s ==--         ",version);
 	mvprintw(5,18,"           By John A. Magliacane           ");
-	mvprintw(6,18,"   KD2BD Software -- Copyright 1991-1999   ");
+	mvprintw(6,18,"   KD2BD Software -- Copyright 1991-2000   ");
 	mvprintw(7,18,"                                           ");
 
 	attrset(COLOR_PAIR(5)|A_BOLD);
@@ -77,13 +81,13 @@ int main()
 		mvprintw(3,18,"                                           ");
 		mvprintw(4,18,"         --== PREDICT  v%s ==--         ",version);
 		mvprintw(5,18,"           By John A. Magliacane           ");
-		mvprintw(6,18,"   KD2BD Software -- Copyright 1991-1999   ");
+		mvprintw(6,18,"   KD2BD Software -- Copyright 1991-2000   ");
 		mvprintw(7,18,"                                           ");
 
 		attrset(COLOR_PAIR(4)|A_BOLD);
 		printw("\n\n\n\n  PREDICT appears to be installed under %s/\n",pwd);
 
-		if (soundcard!=NULL)
+		if (dsp!=-1)
 			printw("  An audio device was found at /dev/dsp");
 		else
 			printw("  No soundcard was found in your system... Bummer!");
@@ -99,7 +103,7 @@ int main()
 
 		fprintf(outfile,"char *predictpath={\"%s/\"}, ",pwd);
 
-		if (soundcard==NULL)
+		if (dsp==-1)
 			fprintf(outfile, "soundcard=0,");
 		else
 			fprintf(outfile, "soundcard=1,");
@@ -111,7 +115,7 @@ int main()
 		printw("\n  Compiling PREDICT...\n");
 		refresh();
 
-		cc=system("cc -Wall -O2 -s -fomit-frame-pointer -I/usr/include/ncurses predict.c -lm -lncurses -o predict");
+		cc=system("cc -Wall -O2 -s -fomit-frame-pointer -I/usr/include/ncurses predict.c -lm -lncurses -lpthread -o predict");
 
 
 		/* Create vocalizer.h */
@@ -122,7 +126,7 @@ int main()
 
 		fclose(outfile);
 
-		if (soundcard && cc==0)
+		if (dsp!=-1 && cc==0)
 		{
 			attrset(COLOR_PAIR(3)|A_BOLD);
 			printw("  Compiling audio capabilities...\n");
