@@ -1,8 +1,14 @@
 /****************************************************************************
 *          PREDICT: A satellite tracking/orbital prediction program         *
-*               Copyright John A. Magliacane, KD2BD 1991-1999               *
-*                       Project started: 26-May-91                          *
-*                         Last update: 02-Jan-00                            *
+*               Copyright John A. Magliacane, KD2BD 1991-2000               *
+*                      Project started: 26-May-1991                         *
+*                        Last update: 16-Jun-2000                           *
+*****************************************************************************
+*         Network sockets added by Ivan Galysh, KD4HBO  10-Jan-2000         *
+*               The network port is 1210.  Protocol is UDP.                 *
+*                    The pthreads library is required.                      *
+*         The socket server is spawned to operate in the background.        *
+*****************************************************************************
 *****************************************************************************
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
@@ -19,12 +25,21 @@
 
 #include <math.h>
 #include <time.h>
+#include <sys/timeb.h>
 #include <curses.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include "predict.h"
 
+#include <pthread.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <fcntl.h>
+
 #ifndef PI
 #define PI 3.141592653569793
 #endif
@@ -55,7 +70,7 @@ char	qthfile[50], tlefile[50], temp[80], output[20], resave=0;
 
 unsigned char val[256];
 
-int	indx, ma256, iaz, iel, isplat, isplong;
+int	indx, ma256, iaz, iel, isplat, isplong, socket_flag=0;
 
 long	i1, i2, i3, de, rv, irk;
 
@@ -66,9 +81,188 @@ double	age, daynum, epoch, sma, range, t1, se, e1, e2, n0, c[4][3],
 	lostime, apogee, perigee, azimuth, ssplat, ssplong, elevation,
 	vk, vm, rm, rk, ak, am, fk, fm, yr, TP=6.283185307179586,
 	PT=1.570796326794897, S1=0.397818675, C1=0.91746406,
-	deg2rad=1.74532925199e-02, R0=6378.16, FF=3.35289186924e-03,
+	deg2rad=1.74532925199e-02, R0=6378.135, FF=3.35289186924e-03,
 	KM=1.609344;
 
+/* The following variables are used by the socket server.  They
+   are updated in the MultiTrack() and SingleTrack() functions. */
+
+char	satn[24][10];
+
+float	az_array[24], el_array[24], long_array[24], lat_array[24],
+	footprint_array[24];
+
+double	doppler100[24];
+
+long	aos_array[24];
+
+unsigned short portbase=0;
+
+
+void bailout(string)
+char *string;
+{
+	/* This function quits ncurses, resets and "beeps"
+	   the terminal, and displays an error message (string)
+	   when we need to bail out of the program in a hurry. */
+
+	beep();	
+	curs_set(1);
+	bkgdset(COLOR_PAIR(1));
+	clear();
+	refresh();
+	endwin();
+	fprintf(stderr,"*** predict: %s!\n",string);
+}
+
+int passivesock (char *service, char *protocol, int qlen)
+{
+	/* This function opens the socket port */
+
+	struct servent *pse;
+	struct protoent *ppe;
+	struct sockaddr_in sin;
+	int s, type;
+	
+	bzero((char *)&sin,sizeof(struct sockaddr_in));
+	sin.sin_family=AF_INET;
+	sin.sin_addr.s_addr=INADDR_ANY;
+	
+	if ((pse=getservbyname(service,protocol)))
+		sin.sin_port=htons(ntohs((unsigned short)pse->s_port)+portbase);
+
+	else if ((sin.sin_port=htons((unsigned short)atoi(service)))==0)
+	{
+		bailout("Can't get service");
+		exit(-1);
+	}
+	
+	if ((ppe=getprotobyname(protocol))==0)
+	{
+		bailout("Can't get protocol");
+		exit(-1);
+	}
+	
+	if (strcmp(protocol,"udp")==0)
+		type=SOCK_DGRAM;
+	else
+		type=SOCK_STREAM;
+	
+	s=socket(PF_INET,type, ppe->p_proto);
+
+	if (s<0)
+	{
+		bailout("Can't open socket");
+		exit(-1);
+	}
+	
+	if (bind(s,(struct sockaddr *)&sin,sizeof(sin))<0)
+	{
+		bailout("Can't bind");
+		exit(-1);
+	}
+	
+	if ((type=SOCK_STREAM && listen(s,qlen))<0)
+	{
+		bailout("Listen fail");
+		exit(-1);
+	}
+
+	return s;
+}
+
+void socket_server()
+{
+	/* This is the socket server code */
+
+	int i, j, sock, alen;
+	struct sockaddr_in fsin;
+	char buf[80], buff[512], satname[26];
+
+	/* Open a socket port at predict */
+	sock=passivesock("predict","udp",10);
+ 	alen=sizeof(fsin);
+	
+	/* This is the main loop for monitoring the socket port */
+
+	while (1)
+	{
+		/* Get datagram from socket port */
+		if (recvfrom(sock,buf,sizeof(buf),0,(struct sockaddr *)&fsin,&alen) < 0)
+			exit(-1);
+
+		/* Parse the command in the datagram */
+		if ((i=strncmp("GET_SAT",buf,7))==0)
+		{
+			/* Get the satellite name */
+			for (i=0; buf[i]!=32 && buf[i]!=0 && i<39; i++);
+
+			for (j=++i; buf[j]!='\n' && buf[j]!=0 && (j-i)<25; j++)
+				satname[j-i]=buf[j];
+
+			satname[j-i]=0;
+
+			/* Do a simple search for the matching satellite name */
+
+			for (i=0; i<24; i++)
+			{
+				/* if ((j=strncmp(satname,satn[i],9))==0) */
+				if ((strncmp(satname,satn[i],9)==0) || (atol(satname)==sat[i].catnum))
+				{
+					/* Build text buffer with satellite data */
+					sprintf(buff,"%s\n%-7.2f\n%+-6.2f\n%-7.2f\n%+-6.2f\n%ld\n%-7.2f\n",satn[i],long_array[i],lat_array[i],az_array[i],el_array[i],aos_array[i],footprint_array[i]);
+
+					/* Send buffer back to the client that sent the request */
+					sendto(sock,buff,strlen(buff),0,(struct sockaddr*)&fsin,sizeof(fsin));
+					break;
+				}
+
+				/* You can do an "else if" statement here to add more commands.
+	 			   Use the same structure as the above "if" statement */
+			}
+		}
+
+		if ((i=strncmp("GET_DOPPLER",buf,11))==0)
+		{
+			/* Get the satellite name */
+			for (i=0; buf[i]!=32 && buf[i]!=0 && i<39; i++);
+
+			for (j=++i; buf[j]!='\n' && buf[j]!=0 && (j-i)<25; j++)
+				satname[j-i]=buf[j];
+
+			satname[j-i]=0;
+
+			/* Do a simple search for the matching satellite name */
+
+			for (i=0; i<24; i++)
+			{
+				if ((strncmp(satname,satn[i],9)==0) || (atol(satname)==sat[i].catnum))
+				{
+					/* Get Normalized (100 MHz)
+					   Doppler shift for sat[i] */
+
+					sprintf(buff,"%s\n%f\n",satn[i],doppler100[i]);
+
+					/* Send buffer back to client who sent request */
+					sendto(sock,buff,strlen(buff),0,(struct sockaddr*)&fsin,sizeof(fsin));
+					break;
+				}
+			}
+		}
+
+		if ((i=strncmp("GET_LIST",buf,8))==0)
+		{
+			buff[0]=0;
+
+			for (i=0; i<24; i++)
+			{
+				sprintf(buff,"%s\n",satn[i]);
+				sendto(sock,buff,strlen(buff),0,(struct sockaddr *)&fsin,sizeof(fsin));	
+			}
+		}
+	} 	
+}
+
 void Banner()
 {
 	curs_set(0);
@@ -80,7 +274,7 @@ void Banner()
 	mvprintw(3,18,"                                           ");
 	mvprintw(4,18,"         --== PREDICT  v%s ==--         ",version);
 	mvprintw(5,18,"           By John A. Magliacane           ");
-	mvprintw(6,18,"   KD2BD Software -- Copyright 1991-1999   ");
+	mvprintw(6,18,"   KD2BD Software -- Copyright 1991-2000   ");
 	mvprintw(7,18,"                                           ");
 }
 
@@ -107,10 +301,10 @@ char *SubString(string,start,end)
 unsigned char *string, start, end;
 {
 	/* This function returns a substring based on the starting
-	   and ending positions provided.  It is used heavily in the
-	   AutoUpdate function when parsing 2-line element data. */
+	   and ending positions provided.  It is used heavily in
+	   the AutoUpdate function when parsing 2-line element data. */
 
-	unsigned x,y;
+	unsigned x, y;
 
 	if (end>=start)
 	{
@@ -176,7 +370,7 @@ char *line1, *line2;
 	   as orbital data based on a simple checksum calculation alone. */
 
 	int x;
-	unsigned char sum1, sum2;
+	unsigned sum1, sum2;
 
 	/* Compute checksum for each line */
 
@@ -188,14 +382,13 @@ char *line1, *line2;
 	  (line1[0]^'1')  | (line1[1]^' ')  | (line1[7]^'U')  |
 	  (line1[8]^' ')  | (line1[17]^' ') | (line1[23]^'.') |
 	  (line1[32]^' ') | (line1[34]^'.') | (line1[43]^' ') |
-	  (line1[52]^' ') | (line1[59]^'-') | (line1[61]^' ') |
-	  (line1[63]^' ') | (line2[0]^'2')  | (line2[1]^' ')  |
-	  (line2[7]^' ')  | (line2[11]^'.') | (line2[16]^' ') |
-	  (line2[20]^'.') | (line2[25]^' ') | (line2[33]^' ') |
-	  (line2[37]^'.') | (line2[42]^' ') | (line2[46]^'.') |
-	  (line2[51]^' ') | (line2[54]^'.') | (line1[2]^line2[2]) |
-	  (line1[3]^line2[3]) | (line1[4]^line2[4]) |
-	  (line1[5]^line2[5]) | (line1[6]^line2[6]) |
+	  (line1[52]^' ') | (line1[61]^' ') | (line1[63]^' ') |
+	  (line2[0]^'2')  | (line2[1]^' ')  | (line2[7]^' ')  |
+	  (line2[11]^'.') | (line2[16]^' ') | (line2[20]^'.') |
+	  (line2[25]^' ') | (line2[33]^' ') | (line2[37]^'.') |
+	  (line2[42]^' ') | (line2[46]^'.') | (line2[51]^' ') |
+	  (line2[54]^'.') | (line1[2]^line2[2]) | (line1[3]^line2[3]) |
+	  (line1[4]^line2[4]) | (line1[5]^line2[5]) | (line1[6]^line2[6]) |
 	  (isdigit(line1[68]) ? 0 : 1) | (isdigit(line2[68]) ? 0 : 1) |
 	  (isdigit(line1[18]) ? 0 : 1) | (isdigit(line1[19]) ? 0 : 1) |
 	  (isdigit(line2[31]) ? 0 : 1) | (isdigit(line2[32]) ? 0 : 1);
@@ -285,7 +478,7 @@ char ReadDataFiles()
 		fclose(fd);
 		flag+=2;
 		resave=0;
-	}	
+	}
 
 	return flag;
 }
@@ -309,9 +502,10 @@ void SaveQTH()
 
 void SaveTLE()
 {
-	int x, y;
 	FILE *fd;	
-	unsigned char sum, line1[80], line2[80], string[20];
+	int x, y;
+	unsigned sum;
+	unsigned char line1[80], line2[80], string[20];
 
  	/* Save orbital data in the form of NASA TLEs */
 
@@ -412,12 +606,11 @@ char *string;
 	   or via the command line.  string==filename of 2-line element
 	   set if this function is invoked via the command line. */
 
-	unsigned char line1[80], line2[80], str0[80], str1[80], str2[80],
-		      filename[50], saveflag=0, interactive=0, savecount=0,
-		      kepcount=0;
+	char line1[80], line2[80], str0[80], str1[80], str2[80],
+	     filename[50], saveflag=0, interactive=0, savecount=0;
 
-	double database_epoch=0.0, tle_epoch=0.0;
-	int i, database_year, tle_year, success=0;
+	float database_epoch=0.0, tle_epoch=0.0, database_year, tle_year;
+	int i, success=0, kepcount=0;
 	FILE *fd;
 
 	do
@@ -449,7 +642,7 @@ char *string;
 		/* Prevent "." and ".." from being used as a
 		   filename otherwise strange things happen. */
 
-		if (strlen(filename)==0 || strcmp(filename,".")==0 || strcmp(filename,"..")==0)
+		if (strlen(filename)==0 || strncmp(filename,".",1)==0 || strncmp(filename,"..",2)==0)
 			return 0;
 
 		fd=fopen(filename,"r");
@@ -485,35 +678,34 @@ char *string;
 					   Copy strings str1 and
 					   str2 into line1 and line2 */
 
-					strncpy(line1,str1,75);     
+					strncpy(line1,str1,75);
 					strncpy(line2,str2,75);
 					kepcount++;
 
 					/* Scan for object number in datafile to see
 					   if this is something we're interested in */
 
-					for (i=0; (i<=24 && sat[i].catnum!=atol(SubString(line1,2,6))); i++);
+					for (i=0; (i<24 && sat[i].catnum!=atol(SubString(line1,2,6))); i++);
 
-					if (i!=25)
+					if (i!=24)
 					{
-						/* We found it.  Check if it needs updating */
+						/* We found it!  Check to see if it's more
+						   recent than the data we already have. */
 
-						database_year=sat[i].year;
+						if (sat[i].year<=50)
+							database_year=365.25*(100.0+(float)sat[i].year);
+						else
+							database_year=365.25*(float)sat[i].year;
 
-						/* Do the Y2K thing... */
+						database_epoch=(float)sat[i].refepoch+database_year;
 
-						if (database_year<=50)
-							database_year+=100;
 
-						database_epoch=((double)database_year*365.25)+sat[i].refepoch;
-						tle_year=atoi(SubString(line1,18,19));
+						tle_year=(float)atof(SubString(line1,18,19));
 
-						/* Do the Y2K thing... */
+						if (tle_year<=50.0)
+							tle_year+=100.0;
 
-						if (tle_year<=50)
-							tle_year+=100;
-
-						tle_epoch=((double)tle_year*365.25)+atof(SubString(line1,20,31));
+						tle_epoch=(float)atof(SubString(line1,20,31))+(tle_year*365.25);
 
 						/* Update only if TLE epoch >= epoch in data file
 						   so we don't overwrite current data with older
@@ -558,7 +750,7 @@ char *string;
 							sat[i].meanmo=atof(SubString(line2,52,62));
 							sat[i].drag=atof(SubString(line1,33,42));
 							sat[i].orbitnum=atof(SubString(line2,63,67));
-						 }
+						}
 					}
 
 					 fgets(str0,75,fd);     	
@@ -693,7 +885,12 @@ double CurrentDaynum()
 	/* Read the system clock and return the number
 	   of days since 31Dec79 00:00:00 UTC (daynum 0) */
 
-	return(((double)time(NULL))/86400.0)-3651.0;
+	struct timeb tptr;
+	int x;
+
+	x=ftime(&tptr);
+
+	return ((((double)tptr.time+0.001*(double)tptr.millitm)/86400.0)-3651.0);
 }
 
 char *Daynum2String(daynum)
@@ -704,12 +901,13 @@ double daynum;
 	   date as a string of the form "Tue 12Oct99 17:22:37". */
 
 	char timestr[25];
-	long x;
+	time_t t;
+	int x;
 
 	/* Convert daynum to Unix time (seconds since 01-Jan-70) */
-	x=(long)rint(86400.0*(daynum+3651.0));
+	t=(time_t)rint(86400.0*(daynum+3651.0));
 
-	sprintf(timestr,"%s",asctime(gmtime(&x)));
+	sprintf(timestr,"%s",asctime(gmtime(&t)));
 
 	if (timestr[8]==' ')
 		timestr[8]='0';
@@ -747,7 +945,7 @@ double GetStartTime()
 	{
 		bkgdset(COLOR_PAIR(2)|A_BOLD);
 		clear();
-		printw("\n\n\n\t     Starting UTC Date And Time For Predictions Of %-15s\n\n",sat[indx].name);
+		printw("\n\n\n\t     Starting UTC Date and Time for Predictions of %-15s\n\n",sat[indx].name);
 		bozo_count++;
 
 		strcpy(string,Daynum2String(CurrentDaynum()));
@@ -756,10 +954,13 @@ double GetStartTime()
 			string[x-4]=string[x];
 
 		attrset(COLOR_PAIR(4)|A_BOLD);
-		printw("\t       Format: %s ----- ",string);
-		printw("Default is `NOW'\n\n\n\n\n\n\n");
+		printw("\t\t    Format: %s -or- ",string);
+		string[7]=0;
+		printw("%s",string);
+		attrset(COLOR_PAIR(2)|A_BOLD);
+		mvprintw(21,30,"Default is `NOW'");
 		attrset(COLOR_PAIR(3)|A_BOLD);
-		printw(" Enter Start Date & Time >> ");
+		mvprintw(13,1,"Enter Start Date & Time >> ");
 		curs_set(1);
 		refresh();
 		echo();
@@ -774,6 +975,20 @@ double GetStartTime()
 			/* Select `NOW' */
 			return(CurrentDaynum());
 
+		if (strlen(line)==7)
+		{
+			line[7]=' ';
+			line[8]='0';
+			line[9]='0';
+			line[10]=':';
+			line[11]='0';
+			line[12]='0';
+			line[13]=':';
+			line[14]='0';
+			line[15]='0';
+			line[16]=0;
+		}
+			
 		/* Check Day */
 		good = (isdigit(line[0]) && isdigit(line[1])) ? 1 : 0;
 
@@ -825,14 +1040,8 @@ double GetStartTime()
 		   correctly after several attempts, then the user
 		   is a "bozo" and obviously can't follow directions. */
 
-		beep();	
-		curs_set(1);
-		bkgdset(COLOR_PAIR(1));
-		clear();
-		refresh();
-		endwin();
-		printf("*** predict: Too Many Errors!\n\n");
-		exit(1);
+		bailout("Too Many Errors");
+		exit(-1);
 	}
 
 	/* Decode Year */
@@ -1042,24 +1251,14 @@ double daynum;
 	mvprintw(23,64,"%+-6.2f  El",el/deg2rad);
 }
 
-void PreCalc(mode)
-char mode;
+double PreCalc(daynum)
+double daynum;
 {
-	/* This function performs preliminary calculations prior to
-	   tracking or prediction.  mode=='p' when in predict mode
-	   to get starting time from user.  mode=='r' in tracking
-	   mode to read the system's current date/time. */
+	/* This function performs preliminary calculations
+	   prior to tracking or prediction. */
  
 	epoch=DayNum(1,0,sat[indx].year)+sat[indx].refepoch;
-
-	if (mode=='p')
-		daynum=GetStartTime();
-
-	if (mode=='r')
-		daynum=CurrentDaynum();
-
 	age=daynum-epoch;
-
 	yr=(float)sat[indx].year;
 
 	/* Do the Y2K thing... */
@@ -1091,6 +1290,8 @@ char mode;
 	y9=r9*cos(l8)*s8;
 	apogee=sma*(1.0+sat[indx].eccn)-R0;
 	perigee=sma*(1.0-sat[indx].eccn)-R0;
+
+	return daynum;
 }
 
 void Calc()
@@ -1159,7 +1360,6 @@ void Calc()
 		azimuth+=360.0;
 
 	ma256=(int)256.0*q;
-
 	am=ak/KM;
 	rk=sqrt(range); 
 	rm=rk/KM;
@@ -1200,7 +1400,23 @@ int x;
 	apogee=sma*(1.0+sat[x].eccn)-R0;
 
 	if ((acos(R0/(apogee+R0))+(lin*deg2rad)) > fabs(qth.stnlat*deg2rad))
-	
+		return 1;
+	else
+		return 0;
+}
+
+char Decayed(x)
+int x;
+{
+	/* This function returns a 1 if it appears that the
+	   satellite pointed to bt 'x' has decayed at the
+	   time of 'daynum' */
+
+	double satepoch;
+
+	satepoch=DayNum(1,0,sat[x].year)+sat[x].refepoch;
+
+	if (satepoch+((16.666666-sat[x].meanmo)/(10.0*fabs(sat[x].drag))) < daynum)
 		return 1;
 	else
 		return 0;
@@ -1225,7 +1441,7 @@ double FindAOS()
 
 	aostime=0.0;
 
-	if (AosHappens(indx) && Geostationary(indx)!=1)
+	if (AosHappens(indx) && Geostationary(indx)==0 && Decayed(indx)==0)
 	{
 		Calc();
 
@@ -1240,7 +1456,7 @@ double FindAOS()
 			   daynum-=0.0007*(elevation*(((ak/8400.0)+0.46))-2.0);
 
 			   but it sometimes skipped passes for
-			   satellites in highly eccentric orbits. */
+			   satellites in highly elliptical orbits. */
 
 			Calc();
 		}
@@ -1310,7 +1526,7 @@ double NextAOS()
 
 	aostime=0.0;
 
-	if (AosHappens(indx) && Geostationary(indx)!=1)
+	if (AosHappens(indx) && Geostationary(indx)==0 && Decayed(indx)==0)
 		daynum=FindLOS2()+0.014;  /* Move to LOS + 20 minutes */
 
 	return (FindAOS());
@@ -1322,7 +1538,7 @@ char *string, mode;
 	/* This function buffers and displays orbital predictions
 	   and allows screens to be saved to a disk file. */
 
-	char type[8], spaces[80], head1[70], head2[70], head3[70];
+	char type[20], spaces[80], head1[70], head2[70], head3[70];
 	int key, ans=0, l, x, t;
 	static char buffer[1450], lines, quit;
 	static FILE *fd;
@@ -1345,6 +1561,9 @@ char *string, mode;
 		if (mode=='v')
 			strcpy(type,"Visual");
 
+		if (mode=='s')
+			strcpy(type,"Solar Illumination");
+
 		l=strlen(qth.callsign)+strlen(sat[indx].name)+strlen(type);
 
 		spaces[0]=0;
@@ -1353,7 +1572,12 @@ char *string, mode;
 			strcat(spaces," ");
 
 		sprintf(head1,"\n%s%s's %s Calendar For %s", spaces, qth.callsign, type, sat[indx].name);
-		sprintf(head2,"\n\t   Date     Time    El   Az  Phase  Lat   Long    Range   Orbit\n");
+
+		if (mode=='s')
+			sprintf(head2,"\n\t  Date     Mins/Day    Sun%c          Date      Mins/Day    Sun%c\n",37,37);
+		else
+			sprintf(head2,"\n\t   Date     Time    El   Az  Phase  Lat   Long    Range   Orbit\n");
+
 		sprintf(head3,"      -----------------------------------------------------------------");
 
 		strcat(buffer,string);
@@ -1363,10 +1587,8 @@ char *string, mode;
 		{
 			bkgdset(COLOR_PAIR(2)|A_BOLD);
 			clear();
-			/* printw("%s",head1); */
 			addstr(head1);
 			attrset(COLOR_PAIR(4)|A_BOLD);
-			/* printw("%s%s",head2,head3); */
 			addstr(head2);
 			addstr(head3);
 			attrset(COLOR_PAIR(3)|A_BOLD);
@@ -1374,7 +1596,6 @@ char *string, mode;
 			if (buffer[0]!='\n')
 				printw("\n");
 
-			/* printw("%s",buffer); */
 			addstr(buffer);
 			attrset(COLOR_PAIR(4)|A_BOLD);
 
@@ -1538,16 +1759,15 @@ char mode;
 	   all passes), or through the PrintVisible() function if
 	   mode=='v' (show only visible passes). */
 
-	/* double lin; */
 	char string[80];
 	int quit=0, lastel=0, breakout=0;
 
-	PreCalc('p');
+	daynum=PreCalc(GetStartTime());
 	clear();
 
 	/* Trap geostationary orbits and passes that cannot occur. */
 
-	if (AosHappens(indx) && Geostationary(indx)==0)
+	if (AosHappens(indx) && Geostationary(indx)==0 && Decayed(indx)==0)
 	{
 		do
 		{
@@ -1568,7 +1788,6 @@ char mode;
 					nodelay(stdscr,TRUE);
 					attrset(COLOR_PAIR(4));
 					mvprintw(23,6,"                 Calculating... Press [ESC] To Quit");
-					refresh();
 
 					/* Allow a way out if this
 					   should continue forever... */
@@ -1607,7 +1826,7 @@ char mode;
 			/* Move to next orbit */
 			daynum=NextAOS();
 
-		}  while (quit==0 && breakout==0);
+		}  while (quit==0 && breakout==0 && AosHappens(indx) && Decayed(indx)==0);
 	}
 
 	else
@@ -1615,7 +1834,7 @@ char mode;
 		bkgdset(COLOR_PAIR(5)|A_BOLD);
 		clear();
 
-		if (AosHappens(indx)==0)
+		if (AosHappens(indx)==0 || Decayed(indx)==1)
 			mvprintw(12,5,"*** Passes for %s cannot occur for your ground station! ***\n",sat[indx].name);
 
 		if (Geostationary(indx)==1)
@@ -1916,11 +2135,11 @@ char speak;
 	   of the satellite being tracked.  If speak=='T', then
 	   the speech routines are enabled. */
 
-	int ans, aosflag=0, losflag=0;
+	int ans, aosflag=0;
 	char going_or_coming=0, command[80], oncethru=0, alarm=0;
 	double pl146=0.0, pl435=0.0, old_clock=0.0, old_range=0.0,
 	       dp=0.0, dt=0.0, doppler146=0.0, doppler435=0.0,
-	       fraction=0.0, oldtime=0.0;
+	       fraction=0.0, oldtime=0.0, nextaos=0.0, lostime=0.0;
 
 	curs_set(0);
 	bkgdset(COLOR_PAIR(3));
@@ -1952,7 +2171,7 @@ char speak;
 	mvprintw(21,22,"Orbit Number:");
 
 	halfdelay(2);
-	PreCalc('r');
+	PreCalc(CurrentDaynum());	
 
 	do
 	{
@@ -1987,7 +2206,7 @@ char speak;
 		mvprintw(13,34,"%0.f",vm);
 		mvprintw(14,34,"%0.f",vk);
 
-		dt=(double)time(NULL)-old_clock;
+		dt=daynum*86400.0-old_clock;
 		dp=(rk*1000.0)-old_range;
 
 		if (dt>0.0)
@@ -1995,7 +2214,7 @@ char speak;
 			fraction=-((dp/dt)/299792458.0);
 			doppler146=fraction*146.0e6;
 			doppler435=fraction*435.0e6;
-			old_clock=(double)time(NULL);
+			old_clock=86400.0*daynum;
 			old_range=rk*1000.0;
 		}
 
@@ -2040,7 +2259,7 @@ char speak;
 			mvprintw(19,25,"---");
 			mvprintw(13,50,"-----");
 			mvprintw(14,49," ----- ");
-			losflag=0;
+			lostime=0.0;
 			alarm=0;
 		}
 
@@ -2075,31 +2294,51 @@ char speak;
 			if (Geostationary(indx))
 				mvprintw(22,22,"Satellite orbit is geostationary");
 
-			else if (AosHappens(indx)==0)
+			else if (AosHappens(indx)==0 || Decayed(indx)==1)
 				mvprintw(22,22,"This satellite never reaches AOS");
 
 			else
 			{
 				daynum+=0.003;  /* Move ahead slightly... */
-				mvprintw(22,22,"Next AOS: %s UTC",Daynum2String(FindAOS()));
+				nextaos=FindAOS();
+				mvprintw(22,22,"Next AOS: %s UTC",Daynum2String(nextaos));
 				aosflag=1;
 			}
 		}
 
-		else if (elevation>0.0 && losflag==0)
+		else if (elevation>0.0 && lostime==0.0)
 		{
 			if (Geostationary(indx))
 				mvprintw(22,22,"Satellite orbit is geostationary");
 
 			else
 			{
-				mvprintw(22,22,"LOS at: %s UTC  ",Daynum2String(FindLOS2()));
-				losflag=1;
+				lostime=FindLOS2();
+				mvprintw(22,22,"LOS at: %s UTC  ",Daynum2String(lostime));
+				nextaos=NextAOS();
 			}
 		} 
 
 		FindMoon(CurrentDaynum());
 
+		/* This is where the variables for the socket server are updated. */
+
+		if (socket_flag)
+		{
+			sprintf(satn[indx],"%s",Abbreviate(sat[indx].name,9));
+			az_array[indx]=azimuth;
+			el_array[indx]=elevation;
+			lat_array[indx]=ssplat;
+			long_array[indx]=ssplong;
+			footprint_array[indx]=fk;
+			doppler100[indx]=doppler146/1.46;
+
+			if (nextaos!=0.0)
+				aos_array[indx]=(long)rint(86400.0*(nextaos+3651.0));
+			else
+				aos_array[indx]=0.0;
+		}
+
 		refresh();
 		ans=tolower(getch());
 
@@ -2111,7 +2350,6 @@ char speak;
 		oncethru=1;
 
 	} while (ans!='q' && ans!=27);
-
 	cbreak();
 }
 
@@ -2123,8 +2361,8 @@ void MultiTrack()
 	   for the Sun and Moon are also displayed. */
 
 	int x, y, z, ans;
-	unsigned char nextsat=0, need2calc=1, satindex[25], inrange[25], sunstat=0;
-	double aos[25], temptime, nextcalctime=0.0;
+	unsigned char need2calc=1, satindex[25], inrange[25], sunstat=0;
+	double aos[25], temptime, nextcalctime=0.0, dt[25], dp[25], old_clock[25], old_range[25], fraction[25], doppler[25];
 
 	curs_set(0);
 	attrset(COLOR_PAIR(6)|A_REVERSE|A_BOLD);
@@ -2139,7 +2377,7 @@ void MultiTrack()
 	attrset(COLOR_PAIR(2)|A_REVERSE);
 	printw(" Satellite  Az   El  Lat  Long  Range  | Satellite  Az   El  Lat  Long  Range   ");
 
-	halfdelay(5);  /* May have to reduce this for SLOW machines... */
+	/* halfdelay(10);  May have to reduce this for SLOW machines... */
 
 	do
 	{
@@ -2162,13 +2400,27 @@ void MultiTrack()
 			if (sat[indx].meanmo!=0.0)
 			{
 				daynum=CurrentDaynum();
-				PreCalc('r');
+				PreCalc(daynum);
 				Calc();
 
+				if (socket_flag)
+				{
+					dt[indx]=daynum*86400.0-old_clock[indx];
+					dp[indx]=(rk*1000.0)-old_range[indx];
+				}
+
 				if (elevation>=0.0)
 				{
 					attrset(COLOR_PAIR(2)|A_BOLD);
 					inrange[indx]=1;
+
+					if (socket_flag)
+					{
+						fraction[indx]=-((dp[indx]/dt[indx])/299792458.0);
+						doppler[indx]=fraction[indx]*100.0e6;
+						old_clock[indx]=86400.0*daynum;
+						old_range[indx]=rk*1000.0;
+					}
 				}
 
 				else
@@ -2194,9 +2446,26 @@ void MultiTrack()
 						sunstat='N';
 						break;
 				}
-				mvprintw(y+6,x,"%-10s%3.0f  %+3.0f  %3.0f   %3.0f %6.0f %c",Abbreviate(sat[indx].name,9),azimuth,elevation,ssplat,ssplong,rk,sunstat);
+				mvprintw(y+6,x,"%-10s%3.0f  %+3.0f  %3.0f   %3.0f %6.0f %c", Abbreviate(sat[indx].name,9),azimuth,elevation,ssplat,ssplong,rk,sunstat);
+
+				/* This is where the variables for the socket server are updated. */
+
+				if (socket_flag)
+				{
+					sprintf(satn[indx],"%s",Abbreviate(sat[indx].name,9));
+					az_array[indx]=azimuth;
+					el_array[indx]=elevation;
+					lat_array[indx]=ssplat;
+					long_array[indx]=ssplong;
+					footprint_array[indx]=fk;
+
+					if (inrange[indx] && (fabs(doppler[indx])>0.5))
+						doppler100[indx]=doppler[indx];
+					else
+						doppler100[indx]=0.0;
+				}
 			}
-		}
+ 		}
 
 		attrset(COLOR_PAIR(6)|A_REVERSE|A_BOLD);
 
@@ -2223,12 +2492,22 @@ void MultiTrack()
 				if (sat[indx].meanmo!=0.0)
 				{
 					daynum=CurrentDaynum();
-					PreCalc('r');
+					PreCalc(daynum);
 
 					if (inrange[indx])
 						aos[indx]=NextAOS();
 					else
 						aos[indx]=FindAOS();
+
+					/* Prepare AOS times for socket export */
+
+					if (socket_flag)
+					{
+						if (aos[indx]!=0.0)
+							aos_array[indx]=(long)rint(86400.0*(aos[indx]+3651.0));
+						else
+							aos_array[indx]=0.0;
+					}
 				}
 			}
 
@@ -2254,17 +2533,21 @@ void MultiTrack()
 			mvprintw(20,31,"---------------");
 			attrset(COLOR_PAIR(3)|A_BOLD);
 
-			for (x=0, y=0; x<21 && y!=3; x++)
+			for (x=0, y=0, z=-1; x<21 && y!=3; x++)
 			{
-				if (Geostationary(x)==0 && AosHappens(x)==1)
+				if (Geostationary(x)==0 && AosHappens(x)==1 && Decayed(x)==0)
 				{
 					mvprintw(y+21,19,"%10s on %s UTC",Abbreviate(sat[(int)satindex[x]].name,9),Daynum2String(aos[x]));
+
+					if (z==-1)
+						z=x;
 					y++;
 				}
 			}
 
-			nextsat=satindex[x-3];
-			nextcalctime=aos[x-3];
+			if (z!=-1)
+				nextcalctime=aos[z];
+
 			need2calc=0;
 		}
 
@@ -2272,6 +2555,7 @@ void MultiTrack()
 		FindMoon(CurrentDaynum());
 
 		refresh();
+		halfdelay(2);  /* Increase if CPU load is too high */
 		ans=tolower(getch());
 
 	} while (ans!='q' && ans!=27);
@@ -2279,6 +2563,101 @@ void MultiTrack()
 	cbreak();
 }
 
+void Illumination()
+{
+	double startday, oneminute, sunpercent;
+	int eclipses, minutes, quit, breakout=0;
+	char string1[80], string2[80], string3[80], datestring[20], count;
+
+	oneminute=1.0/(24.0*60.0);
+
+	daynum=floor(PreCalc(GetStartTime()));
+	startday=daynum;
+	count=0;
+
+	curs_set(0);
+	clear();
+
+	do
+	{
+		attrset(COLOR_PAIR(4));
+		mvprintw(23,6,"                 Calculating... Press [ESC] To Quit");
+		refresh();
+
+		count++;
+		daynum=startday;
+
+		for (minutes=0, eclipses=0; minutes<1440; minutes++)
+		{
+			Calc();
+
+			if (FindSun(daynum,0)==' ')
+				eclipses++;
+
+			daynum=startday+(oneminute*(double)minutes);
+		}
+
+		sunpercent=((double)eclipses)/((double)minutes);
+		sunpercent=100.0-(sunpercent*100.0);
+
+		strcpy(datestring,Daynum2String(startday));
+		datestring[11]=0;
+		sprintf(string1,"      %s    %4d    %6.2f%c",datestring,1440-eclipses,sunpercent,37);
+
+		/* Allow a quick way out */
+
+		nodelay(stdscr,TRUE);
+
+		if (getch()==27)
+			breakout=1;
+
+		nodelay(stdscr,FALSE);
+
+		startday+=18.0;
+
+		daynum=startday;
+
+		for (minutes=0, eclipses=0; minutes<1440; minutes++)
+		{
+			Calc();
+
+			if (FindSun(daynum,0)==' ')
+				eclipses++;
+
+			daynum=startday+(oneminute*(double)minutes);
+		}
+
+		sunpercent=((double)eclipses)/((double)minutes);
+		sunpercent=100.0-(sunpercent*100.0);
+
+		strcpy(datestring,Daynum2String(startday));
+		datestring[11]=0;
+		sprintf(string2,"\t %s    %4d    %6.2f%c\n",datestring,1440-eclipses,sunpercent,37);
+
+		strcat(string3,string1);
+		strcat(string3,string2);
+		quit=Print(string3,'s');
+
+		/* Allow a quick way out */
+
+		nodelay(stdscr,TRUE);
+
+		if (getch()==27)
+			breakout=1;
+
+		nodelay(stdscr,FALSE);
+
+		if (count<18)
+			startday-=17.0;
+		else
+		{
+			count=0;
+			startday+=1.0;
+		}
+	}
+	while (quit!=1 && breakout!=1 && Decayed(indx)==0);
+}
+
 void MainMenu()
 {
 	/* Start-up menu.  Your wish is my command. :-) */
@@ -2290,14 +2669,15 @@ void MainMenu()
 	attrset(COLOR_PAIR(3)|A_BOLD);
 	mvprintw(12,21,"[P]: Predict Satellite Passes");
 	mvprintw(13,21,"[V]: Predict Visible Passes");
-	mvprintw(14,21,"[T]: Single Satellite Tracking Mode");
-	mvprintw(15,21,"[M]: Multi-Satellite Tracking Mode");
-	mvprintw(16,21,"[U]: Update Database From NASA TLEs");
-	mvprintw(17,21,"[E]: Keyboard Edit of Orbital Database");
-	mvprintw(18,21,"[G]: Edit Ground Station Information");
-	mvprintw(19,21,"[D]: Display Satellite Orbital Data");
-	mvprintw(20,21,"[I]: Program Information");
-	mvprintw(21,21,"[Q]: Exit Program");
+	mvprintw(14,21,"[S]: Solar Illumination Predictions");
+	mvprintw(15,21,"[T]: Single Satellite Tracking Mode");
+	mvprintw(16,21,"[M]: Multi-Satellite Tracking Mode");
+	mvprintw(17,21,"[U]: Update Database From NASA TLEs");
+	mvprintw(18,21,"[E]: Keyboard Edit of Orbital Database");
+	mvprintw(19,21,"[G]: Edit Ground Station Information");
+	mvprintw(20,21,"[D]: Display Satellite Orbital Data");
+	mvprintw(21,21,"[I]: Program Information");
+	mvprintw(22,21,"[Q]: Exit Program");
 	refresh();
 }
 
@@ -2385,8 +2765,9 @@ void NewUser()
 int main(argc,argv)
 char argc, *argv[];
 {
-	int x, y, key=0;
-	char temp[80], newuser=0;
+	pthread_t thread;
+	int x, y, z, key=0;
+	char updatefile[80], newuser=0;
 	FILE *qth, *tle;
 
 	
@@ -2395,8 +2776,13 @@ char argc, *argv[];
 	for (x=0; x<=255; val[x]=0, x++);
 	for (x='0'; x<='9'; val[x]=x-'0', x++);
 	val['-']=1;
-	temp[0]=0;
+	updatefile[0]=0;
 
+	temp[0]=0;
+	tlefile[0]=0;
+	qthfile[0]=0;
+
+	y=argc-1;
 
 	/* Set up default qth and tle filenames and paths */
 
@@ -2424,40 +2810,42 @@ char argc, *argv[];
 
 	if (newuser==0)
 	{
-		for (x=1; x<argc; x++)
+		for (x=1; x<=y; x++)
 		{
-			while (argv[x][0]=='-')
+			if (strcmp(argv[x],"-u")==0)
 			{
-				for (y=1; y!=strlen(argv[x]); y++)
-				{
-					switch(tolower(argv[x][y]))
-					{
-						case 'u':
-						strcpy(temp,argv[x+1]);
-						break;
-
-						case 't':
-						strcpy(tlefile,argv[x+1]);
-						break;
-
-						case 'q':
-						strcpy(qthfile,argv[x+1]);
-						break;
-					}
-				}
-				x++;
+				z=x+1;
+				if (z<=y && argv[z][0] && argv[z][0]!='-')
+					strcpy(updatefile,argv[z]);
 			}
+
+			if (strcmp(argv[x],"-t")==0)
+			{
+				z=x+1;
+				if (z<=y && argv[z][0] && argv[z][0]!='-')
+					strcpy(tlefile,argv[z]);
+			}
+
+			if (strcmp(argv[x],"-q")==0)
+			{
+				z=x+1;
+				if (z<=y && argv[z][0] && argv[z][0]!='-')
+					strcpy(qthfile,argv[z]);
+			}
+
+			if (strcmp(argv[x],"-s")==0)
+				socket_flag=1;
 		}
 
-		y=ReadDataFiles();
+		x=ReadDataFiles();
 	}
 	else
-		y=3;
+		x=3;
 
-	if (y==3)
+	if (x==3)
 	{
-		if (temp[0])
-			exit(AutoUpdate(temp));
+		if (updatefile[0])
+			exit(AutoUpdate(updatefile));
 
 		initscr();
 		start_color();
@@ -2481,6 +2869,17 @@ char argc, *argv[];
 			QthEdit();
 		}
 
+		/* Socket activated here. 
+		   Remember that the socket data is only
+		   updated when in the real-time tracking modes. */
+
+		if (socket_flag)
+		{
+			pthread_create(&thread,NULL,(void *)socket_server,NULL);
+			bkgdset(COLOR_PAIR(3));
+			MultiTrack();
+		}
+
 		MainMenu();
 
 		do
@@ -2522,7 +2921,6 @@ char argc, *argv[];
 
 				case 't':
 					indx=Select();
-
 					if (indx!=-1 && sat[indx].meanmo!=0.0)
 						SingleTrack(indx,key);
 					MainMenu();
@@ -2537,6 +2935,16 @@ char argc, *argv[];
 					ProgramInfo();
 					MainMenu();
 					break;
+
+				case 's':
+					indx=Select();
+					if (indx!=-1 && sat[indx].meanmo!=0.0)
+					{
+						Print("",0);
+						Illumination();
+					}
+					MainMenu();
+					break;
 			}
 
 		} while (key!='q' && key!=27 && key!='Q');
@@ -2554,10 +2962,10 @@ char argc, *argv[];
 
 		printf("\n%c",7);
 
-		if (y^1)
+		if (x^1)
 			printf("*** ERROR!  Your QTH file \"%s\" could not be loaded!\n",qthfile);
 
-		if (y^2)
+		if (x^2)
 			printf("*** ERROR!  Your TLE file \"%s\" could not be loaded!\n",tlefile);
 
 		printf("\n");
